'use strict';

var fs = require('fs');
var path = require('path');

var autoprefixer = require('autoprefixer');
var postcss = require('postcss');
var sass = require('node-sass');
var uuid = require('node-uuid');

var Core = require('css-modules-loader-core');
var camelCase = require('postcss-camel-case');

var core = new Core([Core.localByDefault, Core.extractImports, camelCase, Core.scope]);

var prefixer = postcss([autoprefixer({
  remove: false
})]);

/**
 * transform the source into autoprefixed css and save it to the target
 *
 * @param {string} target
 * @param {string} hash
 * @param {string} source
 * @returns {Promise}
 */
var transformStyles = function transformStyles(_ref) {
  var _ref$hash = _ref.hash,
      hash = _ref$hash === undefined ? uuid.v4() : _ref$hash,
      _ref$minify = _ref.minify,
      minify = _ref$minify === undefined ? true : _ref$minify,
      source = _ref.source,
      target = _ref.target;

  if (!source) {
    throw new Error('source file location was not provided');
  }

  if (!target) {
    throw new Error('target file location was not provided');
  }

  var sourceFileExtension = source.split('.').slice(-1)[0];

  var renderedFile = void 0;

  if (sourceFileExtension === 'scss') {
    // if the file is SCSS, use node-sass to compile into a Buffer
    renderedFile = sass.renderSync({
      file: source,
      importer: function importer(url) {
        if (url[0] === '~') {
          url = path.resolve(path.dirname(require.main.filename), 'node_modules', url.substring(1));
        }

        return {
          file: url
        };
      }
    }).css;
  } else {
    // otherwise, read the file as a Buffer
    renderedFile = fs.readFileSync(source, 'utf8');
  }

  // convert the Buffer to a string and prefix via postcss
  var string = renderedFile.toString('utf8');
  var prefixedString = prefixer.process(string).css;

  // load the prefixed CSS and convert via CSS Modules, then create a file with all metadata provided
  return core.load(prefixedString, hash).then(function (_ref2) {
    var exportTokens = _ref2.exportTokens,
        injectableSource = _ref2.injectableSource;

    var css = injectableSource.replace(/\/\*(?:(?!\*\/)[\s\S])*\*\/|[\r\n\t]+/g, '');

    if (minify) {
      css = css.replace(/ {2,}/g, ' ').replace(/ ([{:}]) /g, '$1').replace(/([;,]) /g, '$1').replace(/ !/g, '!');
    }

    // provide the compiled CSS, the id to use in the style tag, and the selector map
    var fileValue = {
      css: css,
      id: hash + '_Scoped_Styles',
      selectors: exportTokens
    };

    // clean the string to mean the rapid7 ESLint config rules
    var bufferString = JSON.stringify(fileValue).replace(/'/g, '\\\'').replace(/"/g, '\'').replace(/,/g, ', ');

    // make a Buffer of an exportable file
    var buffer = new Buffer('module.exports = ' + bufferString + ';\n');

    // write the file to the target
    fs.writeFileSync(target, buffer);

    return fileValue;
  });
};

module.exports = transformStyles;